joypad:
    move.l  #POSH,HANDLERS              ; Use Position button handlers by default
    btst.b  #CNT_A,BIOS_P1CURRENT
    beq     .noa
    move.l  #ZOOMH,HANDLERS             ; A pressed: use Zoom button handlers
.noa:
    btst.b  #CNT_B,BIOS_P1CURRENT
    beq     .nob
    move.l  #NUMH,HANDLERS              ; B pressed: use Sprite number/size button handlers
    bset.b  #0,MODE
    move.l  #TEXT_CONT,d0
    jsr     LoadMess
.nob:
    btst.b  #CNT_C,BIOS_P1CHANGE
    beq     .noc
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    bchg.l  #6,d0                       ; C pressed: toggle bit 6 of SCB3 word (chain on/off)
    move.w  d0,VRAM_RW
.noc:
    btst.b  #CNT_D,BIOS_P1CHANGE
    beq     .nod
    bchg.b  #0,MODE                     ; D pressed: toggle joypad detection mode
    beq     .textstepmode
    move.l  #TEXT_STEP,d0               ; Update fixed text accordingly
    jsr     LoadMess
    bra     .nod
.textstepmode:
    move.l  #TEXT_CONT,d0
    jsr     LoadMess
.nod:

    lea     BIOS_P1CURRENT,a2           ; Choose between BIOS RAM values according to mode
    btst.b  #0,MODE
    beq     .continuous
    lea     BIOS_P1CHANGE,a2
.continuous:

    movea.l HANDLERS,a1                 ; Cycle through up/down/left/right and call subroutine if
    movea.l (a1)+,a0                    ; button is pressed
    clr.l   d0
    move.l  #4-1,d7                     ; 4 buttons to check
.buttonlp:
    btst.b  d0,(a2)
    beq     .nopress
    movem.l d0/a0-a2,-(a7)
    jsr     (a0)
    movem.l (a7)+,d0/a0-a2
.nopress:
    addq.b  #1,d0                       ; next button
    movea.l (a1)+,a0
    dbra    d7,.buttonlp

    rts

ZOOMH:
    dc.l    Z_up                        ; Zoom handlers
    dc.l    Z_down
    dc.l    Z_left
    dc.l    Z_right

POSH:
    dc.l    P_up                        ; Position handlers
    dc.l    P_down
    dc.l    P_left
    dc.l    P_right

NUMH:
    dc.l    N_up                        ; Number/size handlers
    dc.l    N_down
    dc.l    N_left
    dc.l    N_right

Z_up:
    move.w  SPRITEN,d0
    add.w   #SCB2,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    subq.b  #1,d0                       ; Dec vertical zoom
    move.w  d0,VRAM_RW
    jsr     UpdSCB2
    rts
    
Z_down:
    move.w  SPRITEN,d0
    add.w   #SCB2,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0                  ; Inc vertical zoom
    addq.b  #1,d0
    move.w  d0,VRAM_RW
    jsr     UpdSCB2
    rts
    
Z_left:
    move.w  SPRITEN,d0
    add.w   #SCB2,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    subi.w  #$0100,d0                   ; Dec horizontal zoom
    andi.w  #$0FFF,d0
    move.w  d0,VRAM_RW
    jsr     UpdSCB2
    rts

Z_right:
    move.w  SPRITEN,d0
    add.w   #SCB2,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    addi.w  #$0100,d0                   ; Inc horizontal zoom
    andi.w  #$0FFF,d0
    move.w  d0,VRAM_RW
    jsr     UpdSCB2
    rts
    
P_up:
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    addi.w  #1<<7,d0                   ; Inc Y position
    move.w  d0,VRAM_RW
    jsr     UpdSCB3
    rts

P_down:
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    subi.w  #1<<7,d0                   ; Dec Y position
    move.w  d0,VRAM_RW
    jsr     UpdSCB3
    rts

P_left:
    move.w  SPRITEN,d0
    add.w   #SCB4,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    subi.w  #1<<7,d0                   ; Dec X position
    move.w  d0,VRAM_RW
    jsr     UpdSCB4
    rts

P_right:
    move.w  SPRITEN,d0
    add.w   #SCB4,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    addi.w  #1<<7,d0                   ; Inc X position
    move.w  d0,VRAM_RW
    jsr     UpdSCB4
    rts

N_up:
    cmpi.w  #380,SPRITEN
    beq     .noup
    addq.w  #1,SPRITEN                 ; Inc current sprite number
    jsr     UpdAll
.noup:
    rts

N_down:
    cmpi.w  #1,SPRITEN
    beq     .nodn
    subq.w  #1,SPRITEN                 ; Dec current sprite number
    jsr     UpdAll
.nodn:
    rts

N_left:
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    move.w  d0,d1
    andi.w  #$FFC0,d1
    andi.w  #$003F,d0
    tst.w   d0
    beq     .nodn
    subq.b  #1,d0                      ; Dec sprite size (limit to 0)
.nodn:
    or.w    d1,d0
    move.w  d0,VRAM_RW
    jsr     UpdSCB3
    rts

N_right:
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    move.w  d0,d1
    andi.w  #$FFC0,d1
    andi.w  #$003F,d0
    cmp.w   #33,d0
    beq     .noup
    addq.b  #1,d0                      ; Inc sprite size (limit to 33)
.noup:
    or.w    d1,d0
    move.w  d0,VRAM_RW
    jsr     UpdSCB3
    rts
    

UpdSCB2:
    move.w  #$7044+($20*21),VRAM_ADDR
    jsr     WriteHex
    rts

UpdSCB3:
    move.w  #$7044+($20*32),VRAM_ADDR
    jsr     WriteHex
    rts

UpdSCB4:
    move.w  #$7045+($20*5),VRAM_ADDR
    jsr     WriteHex
    rts
    
UpdAll:
    move.w  #$7044+($20*8),VRAM_ADDR
    move.w  SPRITEN,d0
    jsr     WriteHex
    move.w  SPRITEN,d0
    add.w   #SCB2,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    jsr     UpdSCB2
    move.w  SPRITEN,d0
    add.w   #SCB3,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    jsr     UpdSCB3
    move.w  SPRITEN,d0
    add.w   #SCB4,d0
    move.w  d0,VRAM_ADDR
    move.w  VRAM_RW,d0
    jsr     UpdSCB4
    rts
