;Sprite Experimenter for NeoGeo
;CC furrtek 2011
;www.neogeodev.org

    cpu 68000
    supmode on

    INCLUDE "regdefs.asm"

FLAG_DOTASKS         equ 0

                     org RAMSTART

FLAGS:               ds.b 1
MODE:                ds.b 1
HANDLERS:            ds.l 1
SPRITEN:             ds.w 1

    ;INCLUDE "cdheader.asm"
    INCLUDE "header.asm"

Start:
    lea     $10F300,sp
    move.b  d0,REG_DIPSW
    move.w  #7,REG_IRQACK
    clr.b   FLAGS                 ; Don't do anything special in VBlank
    move.w  #$2000,sr             ; Immediately activate interrupts

    move.l  #($F2FF/2)-1,d7       ; Clear user RAM
    lea     RAMSTART,a0
.clram:
    clr.w   (a0)+
    dbra    d7,.clram

    move.w  #FIXMAP,VRAM_ADDR
    move.w  #1,VRAM_MOD
    move.l  #(40*32)-1,d0         ; Clear fix with tile $00
.clfixlp:
    move.w  #$00FF,VRAM_RW
    dbra    d0,.clfixlp

    move.w  #$0000,PALETTES       ; Black
    move.w  #$7FFF,PALETTES+2     ; White

    lea     Palette,a0            ; Load palette 1 for sprites
    lea     PALETTES+32,a1
    move.l  #16-1,d7
.ldpal:
    move.w  (a0)+,(a1)+
    dbra    d7,.ldpal

    move.w  #$0000,BACKDROPCOLOR  ; Black

    ;Load sprite tilemaps
    move.w  #64,VRAM_ADDR
    move.l  #380-1,d7
.setlps:
    move.l  #32-1,d6
    move.l  #1,d0
.setlp:
    move.w  d0,VRAM_RW
    addq.b  #1,d0
    move.w  #$0100,VRAM_RW        ; Palette 1
    dbra    d6,.setlp
    dbra    d7,.setlps

    ;Set sprite zooms
    move.w  #SCB2+1,VRAM_ADDR
    move.l  #380-1,d7
.setz:
    move.w  #$0FFF,VRAM_RW
    dbra    d7,.setz

    ;Set sprites Y position and sizes
    move.w  #SCB3+1,VRAM_ADDR
    move.w  #((496-64)<<7)+32,VRAM_RW       ; Only sprite 1 is 32 tiles high
    move.l  #380-1-1,d7
.sety:
    move.w  #((496-64)<<7),VRAM_RW
    dbra    d7,.sety

    ;Set sprites X position
    move.w  #SCB4+1,VRAM_ADDR
    move.l  #380-1,d7
.setx:
    move.w  #64<<7,VRAM_RW
    dbra    d7,.setx

    ;First usable sprite is #1
    move.w  #1,SPRITEN

    ;Display fixed text
    move.l  #TEXT_LAYOUT,d0
    jsr     LoadMess

    ;Display values
    jsr     UpdAll

    bset.b  #FLAG_DOTASKS,FLAGS

Loop:
   bra     Loop
   
LoadMess:
    bset.b  #0,BIOS_MESS_BUSY
    movea.l BIOS_MESS_POINT,a0  ;Get current pointer in buffer
    move.l  d0,(a0)+
    move.l  a0,BIOS_MESS_POINT  ;Update pointer
    bclr.b  #0,BIOS_MESS_BUSY   ;Ready to go
    jsr     BIOSF_MESS_OUT
    rts


TEXT_LAYOUT:
  dc.w  $0001    ;Bytes, upper = $00, end code = $FE
  dc.w  $00FE
  dc.w  $2002    ;Auto-inc = $20

  dc.w  $0003    ;VRAM address = $7044
  dc.w  $7044
  dc.w  $0007    ;Direct data output
  dc.b  "Sprite #",$FE
  dc.w  $0005    ;+16
  dc.w  16*$20
  dc.w  $0007    ;Direct data output
  dc.b  "SCB2:",$FE
  dc.w  $0005    ;+11
  dc.w  11*$20
  dc.w  $0007    ;Direct data output
  dc.b  "SCB3:",$FE

  dc.w  $0003    ;VRAM address = $7045
  dc.w  $7045
  dc.w  $0007    ;Direct data output
  dc.b  "SCB4:",$FE

  dc.w  $0003    ;VRAM address = $705A
  dc.w  $705A
  dc.w  $0007    ;Direct data output
  dc.b  "Hold A:Zoom",$FF,$FF,$FF,$FF,$FF,"Hold B:Sprite #/Size",$FE
  dc.w  $0005    ;Next line
  dc.w  $0001
  dc.w  $0007    ;Direct data output
  dc.b  "C:Chain sprite",$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,$FF,"D:Step mode",$FE
  dc.w  $0000
  
TEXT_CONT:
  dc.w  $0001    ;Bytes, upper = $00, end code = $FE
  dc.w  $00FE
  dc.w  $0003    ;VRAM address
  dc.w  $705B+(25*$20)
  dc.w  $0007    ;Direct data output
  dc.b  "D:Cont mode",$FE
  dc.w  $0000

TEXT_STEP:
  dc.w  $0001    ;Bytes, upper = $00, end code = $FE
  dc.w  $00FE
  dc.w  $0003    ;VRAM address
  dc.w  $705B+(25*$20)
  dc.w  $0007    ;Direct data output
  dc.b  "D:Step mode",$FE
  dc.w  $0000

Palette:
   include  "spritexp_palettes.inc"

VBlank:
    btst    #7,BIOS_SYSTEM_MODE
    bne     .gamemode
    jmp     BIOSF_SYSTEM_INT1
.gamemode:
    movem.l d0-d7/a0-a6,-(a7)
    move.b  d0,REG_DIPSW          ; Watchdog
    btst.b  #FLAG_DOTASKS,FLAGS
    beq     .nothing

    jsr     BIOSF_SYSTEM_IO

    jsr     joypad

.nothing:
    movem.l (a7)+,d0-d7/a0-a6
    move.w  #4,REG_IRQACK
    rte

    include "joypad.asm"

WriteHex:
    move.l  d1,-(a7)
    move.w  #$20,VRAM_MOD
    move.w  d0,d1
    andi.w  #$F000,d1
    rol.w   #4,d1
    jsr     hexshift
    addi.w  #$30,d1            ; numbers start in fix tileset
    move.w  d1,VRAM_RW         ; MSB high nibble

    move.w  d0,d1
    andi.w  #$0F00,d1
    lsr.w   #8,d1
    jsr     hexshift
    addi.w  #$30,d1
    move.w  d1,VRAM_RW         ; MSB low nibble

    move.w  d0,d1
    andi.w  #$00F0,d1
    lsr.w   #4,d1
    jsr     hexshift
    addi.w  #$30,d1
    move.w  d1,VRAM_RW         ; LSB high nibble

    move.w  d0,d1
    andi.w  #$000F,d1
    jsr     hexshift
    addi.w  #$30,d1
    move.w  d1,VRAM_RW         ; LSB low nibble
    move.l  (a7)+,d1
    rts

hexshift:
    cmp.b   #$A,d1
    blt.b   r
    addq.w  #7,d1
r:
    rts
